package com.zefie.zkiosk

import android.annotation.SuppressLint
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.os.PowerManager
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.webkit.WebResourceRequest
import android.webkit.WebResourceResponse
import android.webkit.WebView
import android.webkit.WebViewClient
import androidx.appcompat.app.AppCompatActivity
import androidx.webkit.WebViewAssetLoader
import android.widget.PopupWindow
import android.widget.RelativeLayout
import android.widget.SeekBar
import android.widget.TextView
import com.google.android.material.floatingactionbutton.FloatingActionButton
import org.videolan.libvlc.util.VLCVideoLayout
import java.util.Locale


class MainActivity : AppCompatActivity() {
    private lateinit var vlcPlayer1: VLCPlayer
    private lateinit var vlcPlayer2: VLCPlayer
    private lateinit var videoLayout1: VLCVideoLayout
    private lateinit var videoLayout2: VLCVideoLayout
    private lateinit var wakeLock: PowerManager.WakeLock
    private lateinit var pauseButton: FloatingActionButton
    private lateinit var switchButton: FloatingActionButton
    private lateinit var restartButton: FloatingActionButton
    private lateinit var wxWebView: WebView
    private lateinit var clockWebView: WebView
    private lateinit var popupWindow: PopupWindow
    private lateinit var popupText: TextView
    private var switchSeconds: Long = 10
    private val handler = android.os.Handler()
    private var showingFirst = true
    private var switcherPaused = false

    @SuppressLint("SetJavaScriptEnabled")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        goFullscreen()
        supportActionBar?.hide()


        switchButton = findViewById(R.id.switchButton)
        pauseButton = findViewById(R.id.pauseButton)
        videoLayout1 = findViewById(R.id.videoLayout1)
        videoLayout2 = findViewById(R.id.videoLayout2)
        restartButton = findViewById(R.id.restartButton)
        vlcPlayer1 = VLCPlayer(this, videoLayout1, 1)
        vlcPlayer2 = VLCPlayer(this, videoLayout2, 2)
        val seekBar = findViewById<SeekBar>(R.id.durationSlider)
        setupSliderPopup(seekBar)

        wxWebView = findViewById(R.id.wxWebView)
        clockWebView = findViewById(R.id.clockWebView)

        videoLayout1.postDelayed({ toggleVideoLayouts() }, 100)
        videoLayout1.postDelayed({ toggleVideoLayouts() }, 500)

        wxWebView.settings.loadWithOverviewMode = true
        wxWebView.settings.useWideViewPort = true
        wxWebView.settings.builtInZoomControls = false
        wxWebView.settings.displayZoomControls = false
        wxWebView.settings.javaScriptEnabled = true
        wxWebView.settings.cacheMode = android.webkit.WebSettings.LOAD_NO_CACHE

        clockWebView.settings.loadWithOverviewMode = true
        clockWebView.settings.useWideViewPort = true
        clockWebView.settings.builtInZoomControls = false
        clockWebView.settings.displayZoomControls = false
        clockWebView.settings.javaScriptEnabled = true

        val assetLoader = WebViewAssetLoader.Builder()
            .addPathHandler("/assets/", WebViewAssetLoader.AssetsPathHandler(this))
            .addPathHandler("/res/", WebViewAssetLoader.ResourcesPathHandler(this))
            .build()

        wxWebView.webViewClient = object : WebViewClient() {
            override fun shouldInterceptRequest(
                view: WebView,
                request: WebResourceRequest
            ): WebResourceResponse? {
                return assetLoader.shouldInterceptRequest(request.url)
            }
        }

        clockWebView.webViewClient = object : WebViewClient() {
            override fun shouldInterceptRequest(
                view: WebView,
                request: WebResourceRequest
            ): WebResourceResponse? {
                return assetLoader.shouldInterceptRequest(request.url)
            }
        }

        wxWebView.loadUrl("https://appassets.androidplatform.net/assets/radar.html")
        clockWebView.loadUrl("https://appassets.androidplatform.net/assets/clock.html")

        startPlayer(1)
        startPlayer(2)

        pauseButton.setOnClickListener() {
            onPauseButtonPressed()
        }

        switchButton.setOnClickListener {
            onSwitchButtonPressed()
        }

        restartButton.setOnClickListener {
            restartCurrentStream()
        }

        switchSeconds = seekBar.progress.toLong();

        startVisibilityToggle()

        acquireWakeLock()
    }

    private fun setupSliderPopup(seekBar: SeekBar) {
        val inflater = LayoutInflater.from(this)
        val popupView = inflater.inflate(R.layout.popup_slider_value, null)
        popupText = popupView.findViewById(R.id.popupText)

        popupWindow = PopupWindow(
            popupView,
            WindowManager.LayoutParams.WRAP_CONTENT,
            WindowManager.LayoutParams.WRAP_CONTENT,
            false
        )

        popupWindow.isOutsideTouchable = false
        popupWindow.isTouchable = false
        popupWindow.elevation = 8f

        seekBar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onStartTrackingTouch(seekBar: SeekBar) {
                updatePopup(seekBar)
                popupWindow.showAtLocation(seekBar, Gravity.NO_GRAVITY, 0, 0)

            }

            override fun onProgressChanged(seekBar: SeekBar, progress: Int, fromUser: Boolean) {
                if (fromUser) {
                    updatePopup(seekBar)
                }
                switchSeconds = progress.toLong()
                handler.removeCallbacks(toggleRunnable) // Stop the scheduled toggling
                handler.postDelayed(toggleRunnable, switchSeconds * 1000) // Restart auto-toggle after 10 seconds
            }

            override fun onStopTrackingTouch(seekBar: SeekBar) {
                popupWindow.dismiss()
            }
        })
    }

    private fun updatePopup(seekBar: SeekBar) {
        val progress = seekBar.progress
        popupText.text = String.format(Locale.US, "%d seconds", progress)

        // Get SeekBar thumb position
        val location = IntArray(2)
        seekBar.getLocationOnScreen(location)

        val sliderLeft = location[0]
        val sliderTop = location[1]

        val availableWidth = seekBar.width - seekBar.paddingLeft - seekBar.paddingRight
        val thumbX = sliderLeft + seekBar.paddingLeft + (availableWidth * progress / seekBar.max)
        val thumbY = sliderTop - popupWindow.contentView.measuredHeight - 20 // 20px above the thumb

        popupWindow.contentView.measure(
            View.MeasureSpec.UNSPECIFIED,
            View.MeasureSpec.UNSPECIFIED
        )

        val popupWidth = popupWindow.contentView.measuredWidth
        val xOffset = thumbX - popupWidth / 2

        popupWindow.update(xOffset, thumbY, -1, -1)
    }

    private fun goFullscreen() {
        val decorView = window.decorView
        val uiOptions: Int =
            View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY or
                    View.SYSTEM_UI_FLAG_FULLSCREEN or
                    View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
        decorView.systemUiVisibility = uiOptions
    }

    @SuppressLint("AuthLeak")
    private fun startPlayer(index: Int) {
        if (index == 1) {
            vlcPlayer1.stop()
            vlcPlayer1.playRTSP("rtsp://192.168.11.82/video1")
        }
        if (index == 2) {
            vlcPlayer2.stop()
            vlcPlayer2.playRTSP("rtsp://viewer:viewer@192.168.11.81/video1")
        }
    }

    private fun restartCurrentStream() {
        // Restart the visible stream by stopping and starting it again
        if (showingFirst) {
            startPlayer(1)
        } else {
            startPlayer(2)
        }
    }

    private fun acquireWakeLock() {
        val powerManager = getSystemService(POWER_SERVICE) as PowerManager
        // Create a wake lock with the full-screen flag and the CPU flag
        wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK or PowerManager.ACQUIRE_CAUSES_WAKEUP, "MyApp:WakeLock")
        wakeLock.acquire()  // Acquiring the wake lock
    }

    private fun startVisibilityToggle() {
        handler.postDelayed(toggleRunnable, switchSeconds * 1000) // Start after 10 seconds
    }

    private val toggleRunnable = object : Runnable {
        override fun run() {
            toggleVideoLayouts()
            handler.postDelayed(this, switchSeconds * 1000) // Toggle every 10 seconds
        }
    }

    private fun toggleVideoLayouts() {
        val layout1 = findViewById<View>(R.id.relativeView1)
        val layout2 = findViewById<View>(R.id.relativeView2)

        layout1.post {
            // Save current positions
            val x1 = layout1.x
            val y1 = layout1.y
            val x2 = layout2.x
            val y2 = layout2.y

            // Swap positions
            layout1.x = x2
            layout1.y = y2
            layout2.x = x1
            layout2.y = y1

            // Get current layout params
            val params1 = layout1.layoutParams as RelativeLayout.LayoutParams
            val params2 = layout2.layoutParams as RelativeLayout.LayoutParams

            // Swap width & height manually (not whole objects)
            val tempWidth = params1.width
            val tempHeight = params1.height

            params1.width = params2.width
            params1.height = params2.height

            params2.width = tempWidth
            params2.height = tempHeight

            // Swap margins
            val tempMargins = Rect(
                params1.leftMargin,
                params1.topMargin,
                params1.rightMargin,
                params1.bottomMargin
            )

            params1.setMargins(
                params2.leftMargin,
                params2.topMargin,
                params2.rightMargin,
                params2.bottomMargin
            )

            params2.setMargins(
                tempMargins.left,
                tempMargins.top,
                tempMargins.right,
                tempMargins.bottom
            )

            layout1.layoutParams = params1
            layout2.layoutParams = params2

            if (layout1.x < 0) {
                layout1.x = 0f
            }

            layout1.requestLayout()
            layout2.requestLayout()
        }
        showingFirst = !showingFirst
    }


    private fun onPauseButtonPressed() {
        if (switcherPaused) {
            handler.postDelayed(toggleRunnable, switchSeconds * 1000)
            pauseButton.setImageResource(android.R.drawable.ic_media_pause)
        } else {
            handler.removeCallbacks(toggleRunnable)
            pauseButton.setImageResource(android.R.drawable.ic_media_play)
        }
        switcherPaused = !switcherPaused
    }
    
    // Call this function when the switch button is pressed
    private fun onSwitchButtonPressed() {
        if (!switcherPaused) {
            handler.removeCallbacks(toggleRunnable) // Stop the scheduled toggling
        }
        toggleVideoLayouts() // Instantly switch layouts
        if (!switcherPaused) {
            handler.postDelayed(toggleRunnable, switchSeconds * 1000) // Restart auto-toggle after 10 seconds
        }
    }

    override fun onPause() {
        super.onPause()
        wakeLock.release()
    }

    override fun onResume() {
        super.onResume()
        acquireWakeLock()
    }
    override fun onDestroy() {
        super.onDestroy()
        vlcPlayer1.release()
        vlcPlayer2.release()
        handler.removeCallbacksAndMessages(null)
        wakeLock.release()
    }
}
